@@ -1,4 +1,4 @@
-from six import BytesIO, text_type
+from six import BytesIO, text_type, string_types
 
 from django.http import HttpResponse
 from django.contrib.contenttypes.models import ContentType
@@ -12,6 +12,7 @@ import re
 from collections import namedtuple
 from decimal import Decimal
 from numbers import Number
+import datetime
 
 from report_utils.model_introspection import (
     get_relation_fields_from_model,
@@ -60,7 +61,9 @@ class DataExportMixin(object):
                 # If item is a regular string
                 if isinstance(item, str):
                     # Change it to a unicode string
-                    row[i] = text_type(item, "UTF-8")
+                    row[i] = text_type(item)
+                elif type(item) is dict:
+                    row[i] = text_type(item)
             try:
                 ws.append(row)
             except ValueError as e:
@@ -212,8 +215,14 @@ class DataExportMixin(object):
                     append_display_total(display_totals, display_field, display_field_key)
             else:
                 message += "You don't have permission to " + display_field.name
-        if user.has_perm(model_class._meta.app_label + '.change_' + model_class._meta.model_name) \
-        or user.has_perm(model_class._meta.app_label + '.view_' + model_class._meta.model_name):
+
+        try:
+            model_name = model_class._meta.model_name
+        except AttributeError:
+            model_name = model_class._meta.module_name # needed for Django 1.4.* (LTS)
+
+        if user.has_perm(model_class._meta.app_label + '.change_' + model_name) \
+        or user.has_perm(model_class._meta.app_label + '.view_' + model_name):
 
             def increment_total(display_field_key, display_totals, val):
                 if display_totals.has_key(display_field_key):
@@ -231,7 +240,11 @@ class DataExportMixin(object):
             for position, property_path in property_list.items():
                 property_root = property_path.split('__')[0]
                 root_class = model_class
-                property_root_class = getattr(root_class, property_root)
+                try:
+                    property_root_class = getattr(root_class, property_root)
+                # django-hstore schema compatibility
+                except AttributeError:
+                    continue
                 if type(property_root_class) == ReverseManyRelatedObjectsDescriptor:
                     display_field_paths.insert(1, '%s__pk' % property_root)
                     m2m_relations.append(property_root)
@@ -314,21 +327,25 @@ class DataExportMixin(object):
                     if preview and len(filtered_report_rows) == 50:
                         break
             if hasattr(display_fields, 'filter'):
-                sort_fields = display_fields.filter(sort__gt=0).order_by('-sort').\
-                    values_list('position', 'sort_reverse')
+                sort_fields = display_fields.filter(
+                    sort__gt=0,
+                ).order_by('-sort').values_list('position', 'sort_reverse')
                 for sort_field in sort_fields:
+                    sort_value = sort_field[0]
                     try:
                         filtered_report_rows = sorted(
                             filtered_report_rows,
-                            key=lambda x: self.sort_helper(x, sort_field[0]-1),
+                            key=lambda x: self.sort_helper(x, sort_value),
                             reverse=sort_field[1]
-                            )
-                    except TypeError: # Sorry crappy way to determine if date is being sorted
+                        )
+                    # Crappy way to deal with null dates.
+                    except TypeError:
                         filtered_report_rows = sorted(
                             filtered_report_rows,
-                            key=lambda x: self.sort_helper(x, sort_field[0]-1, date_field=True),
+                            key=lambda x: self.sort_helper(
+                                x, sort_value, date_field=True),
                             reverse=sort_field[1]
-                            )
+                        )
             values_and_properties_list = filtered_report_rows
         else:
             values_and_properties_list = []
@@ -417,7 +434,11 @@ class DataExportMixin(object):
             result = datetime.date(datetime.MINYEAR, 1, 1)
         else:
             result = x[sort_key]
-        return result.lower() if isinstance(result, basestring) else result
+        if isinstance(result, string_types):
+            return result.lower()
+        elif result is None:
+            return ''
+        return result
 
 
 class GetFieldsMixin(object):
@@ -485,23 +506,26 @@ class GetFieldsMixin(object):
 
     def get_related_fields(self, model_class, field_name, path="", path_verbose=""):
         """ Get fields for a given model """
-        field = model_class._meta.get_field_by_name(field_name)
-        if field[2]:
-            # Direct field
-            new_model = field[0].related.parent_model()
+        if field_name:
+            field = model_class._meta.get_field_by_name(field_name)
+            if field[2]:
+                # Direct field
+                new_model = field[0].related.parent_model()
+            else:
+                # Indirect related field
+                new_model = field[0].model()
+
+            if path_verbose:
+                path_verbose += "::"
+            path_verbose += field[0].name
+
+            path += field_name
+            path += '__'
         else:
-            # Indirect related field
-            new_model = field[0].model()
+            new_model = model_class
 
         new_fields = get_relation_fields_from_model(new_model)
         model_ct = ContentType.objects.get_for_model(new_model)
 
-        if path_verbose:
-            path_verbose += "::"
-        path_verbose += field[0].name
-
-        path += field_name
-        path += '__'
-
         return (new_fields, model_ct, path)
 
